package org.planet.snap;

import dermi.*;
import dermi.annotation.*;
import dermi.exception.RemoteException;

import java.util.Properties;
import java.util.Collection;

@DermiRemoteInterface
public interface IApplication extends ERemote {
  // Method signature for the anycall method pair must be the same, except for the return result,
  // which needs to be boolean for the condition method
  @RemoteMethod (granularity = Granularity.ANYCALL)
  public String getAppInstance (String appP2PUrl) throws RemoteException;

  @RemoteMethod (granularity = Granularity.ANYCALL_CONDITION)
  public boolean getAppInstanceCondition (String appP2PUrl) throws RemoteException;

  @RemoteMethod (granularity = Granularity.MULTICALL, type = SynchronyType.SYNCHRONOUS)
  public Properties redirectToSnapApp (String p2pUrl, String webAppDir, IApplication snapSkel) throws ApplicationDeploymentException, RemoteException;

  // When doing a direct call, the object's destination NodeHandle must be passed.
  // IT MUST ALWAYS BE the FIRST PARAMETER in the direct call so as it works correctly.
  @RemoteMethod (granularity = Granularity.DIRECTCALL, type = SynchronyType.SYNCHRONOUS)
  public void startNewDatabaseInstance (dermi.core.NodeHandle nh, String dbGroup, int clusterSize) throws RemoteException;

  // When doing a direct call, the object's destination NodeHandle must be passed.
  // IT MUST ALWAYS BE the FIRST PARAMETER in the direct call so as it works correctly.
  @RemoteMethod (granularity = Granularity.DIRECTCALL, type = SynchronyType.SYNCHRONOUS)
  public Object deploySnapAppReplica (dermi.core.NodeHandle nh, String p2pUrl, String webAppDir) throws ApplicationDeploymentException, RemoteException;

  @RemoteMethod (granularity = Granularity.DIRECTCALL, type = SynchronyType.SYNCHRONOUS)
  public boolean isDatabaseAlive (dermi.core.NodeHandle nh) throws RemoteException;

  //////////////////////
  // INTROSPECTION API
  //////////////////////

  @RemoteMethod (granularity = Granularity.MULTICALL, type = SynchronyType.SYNCHRONOUS)
  public Collection<String> getInstances() throws RemoteException;

  @RemoteMethod (granularity = Granularity.MULTICALL, type = SynchronyType.SYNCHRONOUS)
  public Collection<String> getDatabaseInstances() throws RemoteException;
}
