package org.planet.snap.deployer;

import java.io.File;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class TransformChooserDialog extends TitleAreaDialog {
  private Text p2pUrl;
  private Text appCtx;
  private Text text;
  private Text passwd;
  private Button next;
  private SnapDeployer win;

  public TransformChooserDialog (Shell parentShell, SnapDeployer wd) {
    super (parentShell);
    win = wd;
  }

  protected Control createDialogArea (Composite parent) {
    Composite area = (Composite) super.createDialogArea(parent);
    Composite container = new Composite(area, SWT.NONE);
    container.setLayoutData(new GridData(GridData.FILL_BOTH));

    super.getShell().setText ("Snap Deployer");
    Image img = new Image (super.getShell().getDisplay(), "./images/snap_installer.jpg");
    super.getShell().setImage (img);

    setTitle ("SNAP Web Application Signer and Deployer Wizard");
    setMessage ("Choose web application directory to be packed, signed and deployed into SNAP.");
    setTitleImage (img);

    final Label label = new Label(container, SWT.NONE);
    label.setBounds(15, 17, 450, 28);
    label.setText ("Please choose the web application's directory to be packed, signed and deployed into SNAP, as well as its\nP2P URL identifier:");

    text = new Text(container, SWT.BORDER);
    text.setBounds(118, 63, 272, 20);

    final Label label_1 = new Label(container, SWT.NONE);
    label_1.setBounds (47, 65, 92, 20);
    label_1.setText ("Webapp dir:");

    final Button browseButton = new Button(container, SWT.NONE);
    browseButton.setBounds(396, 62, 54, 23);
    browseButton.setText("Browse...");
    browseButton.addSelectionListener (new SelectionAdapter() {
      public void widgetSelected (SelectionEvent event) {
        String value = onBrowse_click();
        if (value != null) {
          text.setText (value);
          next.setEnabled (true);
        }
      }
    });

    final Label label_2 = new Label(container, SWT.NONE);
    label_2.setBounds (22, 92, 92, 15);
    label_2.setText ("Application's URL:");

    p2pUrl = new Text(container, SWT.BORDER);
    p2pUrl.setText ("p2p://");
    p2pUrl.setBounds (118, 90, 272, 20);

    final Label label_5 = new Label(container, SWT.NONE);
    label_5.setBounds (40, 117, 70, 15);
    label_5.setText ("App. context:");

    appCtx = new Text(container, SWT.BORDER);
    appCtx.setBounds (118, 115, 272, 20);

    final Label label_3 = new Label(container, SWT.NONE);
    label_3.setBounds (25, 144, 92, 15);
    label_3.setText ("Admin Password:");

    passwd = new Text(container, SWT.BORDER);
    passwd.setBounds (118, 142, 272, 20);

    final Label label_4 = new Label(container, SWT.SEPARATOR | SWT.HORIZONTAL);
    label_4.setBounds (7, 200, 485, 5);

    return area;
  }

  protected String onBrowse_click () {
    try {
      DirectoryDialog dChooser = new DirectoryDialog (getShell(), SWT.OPEN);
      return dChooser.open();
    }
    catch (Exception ex) {
      ex.printStackTrace();
      return null;
    }
  }

  protected void createButtonsForButtonBar(Composite parent) {
    createButton (parent, IDialogConstants.BACK_ID, IDialogConstants.BACK_LABEL, false);

    createButton (parent, IDialogConstants.NEXT_ID, IDialogConstants.NEXT_LABEL,
                  true);
    createButton (parent, IDialogConstants.CANCEL_ID,
                  IDialogConstants.CANCEL_LABEL, false);

    Button cancel = getButton (IDialogConstants.CANCEL_ID);
    cancel.addSelectionListener (new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        close();
      }
    });

    Button back = getButton (IDialogConstants.BACK_ID);
    back.setEnabled (true);
    back.addSelectionListener (new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        close();
        win.openKeyGenDialog (getShell().getLocation(), false);
      }
    });

    next = getButton (IDialogConstants.NEXT_ID);
    next.setEnabled (false);
    next.addSelectionListener (new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        String directory = text.getText();
        String appContext = appCtx.getText();
        String p2p = p2pUrl.getText();
        String password = passwd.getText();

        // Transform this webapp into a Snappable WAR file
        try {
          win.transformIntoSnapApp (directory, appContext);
        } catch (Exception e) { e.printStackTrace(); }
        Point p = getShell().getLocation();
        close();
        // Comprovar que p2pUrl es valida (fins i tot mirar si bound, etc)
        win.openSignAndDeployWarFileDialog (p, "." + File.separator + "storage" + File.separator + appContext + ".war", p2p, password);
      }
    });
  }

  protected Point getInitialSize() {
    return new Point(500, 375);
  }
}
